/***********************************
*   EJERCICIO DEL MOTOR
*   Estimadores de estado
*     CLASE  TEORÍA
************************************/
xdel(winsid());
clear all;
clc;

// VALORES
Ra=2;
La=5e-3;
J=5e-3;
B1=5e-3;
Kt=0.7;
Kw=0.8;
Ai=1;
Ts=0.001;

// MATRICES
A=[-Ra/La -Kw/La 0; Kt/J -B1/J 0; 0 1 0];
B=[Ai/La; 0; 0];
C=[1 0 0; 0 1 0; 0 0 1];
D=[0; 0; 0];
// MODELO DE ESTADO CONTINUO

P1=syslin("c",A,B,C,D)

// SIMULACIÓN
T = 0:0.001:0.1;
Y=csim('step',T,P1);
// GRÁFICO VARIABLES DE ESTADO A LAZO CERRADO
xdel(winsid());
subplot(3,1,1)
plot2d(T,Y(1,:),color("red"));
xgrid(color("gray"))
title('Respuesta de las variables de estado')
ylabel("Corriente");
subplot(3,1,2)
plot2d(T,Y(2,:),color("scilabgreen4"));
xgrid(color("gray"))
ylabel("Velocidad");
subplot(3,1,3)
plot2d(T,Y(3,:),color("blue"));
xgrid(color("gray"))
ylabel("Posición");

// Transformación a Modelo Discreto
C1=[0 0 1];
D1=0;

P1=syslin("c",A,B,C1,D1);
Ts=0.001;

P2=dscr(P1,Ts)
Ad=P2.a;
Bd=P2.b;
Cd=P2.c;
Dd=P2.d;
G2=ss2tf(P2)
autov=spec(Ad)

// OBSERVABILIDAD
V=obsv_mat(P2)
det(V)
rank(V)

// CÁLCULO DEL MODELO CANÓNICO OBSERVABLE
clc;
den=coeff(G2.den)
num=coeff(G2.num)
Ac=[0,0,-den(1);1,0,-den(2);0,1,-den(3)];
Cc=[0,0,1];
Bc=[num(1);num(2);num(3)];
Dc=0;
str1=sprintf(' Modelo Canónico Observable ');
disp(str1);
P3=syslin("d",Ac,Bc,Cc,Dc)
Vco=obsv_mat(P3)
det(Vco)
str1=sprintf('Matriz de Transformación al MCO');
disp(str1);
Pco=V^-1*Vco
Pco_1=Pco^-1

// MATRIZ A TRANSFORMADA

Aco=Pco_1*Ad*Pco
Cco=Cd*Pco

// TRANSFORMACIÓN DEL MODELO 
P4=ss2ss(P2,Pco)

// CÁLCULO DEL ESTIMADOR
clc;
// AUTOVALORES
Z1=0;
Z2=0;
Z3=0;
//Z1=0.94;
//Z2=0.72;
//Z3=0.82;
autov_d=[Z1,Z2,Z3];
// CÁLCULO DEL VECTOR DE REALIMENTACIÓN DEL ESTIMADOR

Pcr=poly(autov_d,"z","roots");    // Polinomio deseado
Pcm=poly(autov,"z","roots");      // Polinomio de la planta
Pcd=Pcr-Pcm;                 // Diferencia
Hco=(coeff(Pcd))'              // Vector de realimentación del Mco

Hco=(ppol(Aco',Cco',autov_d))' // Vector de realimentación H del Mco

H=Pco*Hco

H=(ppol(Ad',Cd',autov_d))' // Vector de realimentación H

// MATRICES DEL ESTIMADOR
Ae=P2.a-H*P2.c
autov_e=spec(Ae)
Be=[P2.b,H]
Ce=eye(3,3)
Xe0=[1;1;1]

// MODELO DEL ERROR
xdel(winsid());
clc;
Ve=zeros(3,1); 
Pe=syslin("d",Ae,Ve,Ce,Ve,Xe0)
// SIMULACIÓN
T = 0:Ts:0.1;
u=zeros(1,length(T));
Y=dsimul(Pe,u);
// GRÁFICO DEL ERROR
xdel(winsid());
subplot(3,1,1)
plot2d(T,Y(1,:),color("red"));
xgrid(color("gray"))
title('Respuesta de las variables de estado')
ylabel("Corriente");
subplot(3,1,2)
plot2d(T,Y(2,:),color("scilabgreen4"));
xgrid(color("gray"))
ylabel("Velocidad");
subplot(3,1,3)
plot2d(T,Y(3,:),color("blue"));
xgrid(color("gray"))
ylabel("Posición");

// RESPUESTA DEL ESTIMADOR 

At=[Ad,zeros(3,3);H*Cd,Ae]
Bt=[Bd;Bd]
Ct=eye(6,6)
Dt=zeros(6,1)
Xt=[Xe0;zeros(3,1)]
Pt=syslin("d",At,Bt,Ct,Dt,Xt)

// SIMULACIÓN
T = 0:Ts:0.1;
u=ones(1,length(T));
Y=dsimul(Pt,u);
// GRÁFICO DEL ERROR
xdel(winsid());
subplot(3,1,1)
plot2d(T,Y(1,:),color("red"));
plot2d(T,Y(4,:),color("blue"));
xgrid(color("gray"))
title('Respuesta de las variables de estado')
ylabel("Corriente");
subplot(3,1,2)
plot2d(T,Y(2,:),color("red"));
plot2d(T,Y(5,:),color("blue"));
xgrid(color("gray"))
ylabel("Velocidad");
subplot(3,1,3)
plot2d(T,Y(3,:),color("red"));
plot2d(T,Y(6,:),color("blue"))
xgrid(color("gray"))
ylabel("Posición");

//PARÁMETROS PARA EL CÁLCULO DE LA REALIMENTACIÓN

// Se plantea un sistema con una transferencia a lazo cerrado
// con un Psi=0.6 y Wn=12 r/s.

Psi= 0.6;
Wn = 12;
p1 = -Psi*Wn+%i*Wn*sqrt(1-Psi^2)
p2 = -Psi*Wn-%i*Wn*sqrt(1-Psi^2)

// AGREGO El AUTOVALOR RESTANTES CON PARTE REAL DE AL MENOS 
// 10 VECES MÁS GRANDE EN MÓDULO

p3 = 10*(-Psi*Wn)

// TRANSFORMO ESTOS AUTOVALORES AL PLANO Z

Polos= [exp(p1*Ts) exp(p2*Ts) exp(p3*Ts)]

// DISEÑO DE LA REALIMENTACIÓN DE ESTADOS

Kt1=ppol(P2.a,P2.b,Polos)

Ar=(Ad-Bd*Kt1);
Br=Bd;
Cr=[0 0 1];
autov_r=spec(Ar)

Klc=Cr*(eye(3)-Ar)^-1*Br
A0=1/Kt1(3)

Kt0=Kt1*A0

K1=Kt0(1)
K2=Kt0(2)
K3=Kt0(3)

// Cálculo del H para norma mínima
clc;
xdel(winsid());
Nmin=1e23;
zmin=[0 0 0];
for z1=0.5:0.02:1
    for z2=0.5:0.02:1
        for z3=0.5:0.02:1
            z=[z1,z2,z3];
            H1=(ppol(P2.a',P2.c',z))';
            N=norm(H1,2);
            if N < Nmin
                Nmin = N;
                zmin = z;
                Hmin = H1;
            end 
        end
    end
end
zmin
Hmin
Nmin
